﻿//******************************************************************************
//Name          : XYZDisplay.cs
//Description   : Implementation to experience the beauty and the pitfalls of Visualization
//Author        : Dr. Oliver Kluge, 90411 Nuernberg, Germany
//Copyright(C)  : Oliver Kluge 2014 
//------------------------------------------------------------------------------
//******************************************************************************

using System.Drawing;
using System.Drawing.Drawing2D;
using System.Windows.Forms;


namespace Presentation
{
    public class MyXYZDisplay : Form
    {
        const int numOfLabels = 2*5;    // labels for the x-axis and for the y-axis
        const int numOfColors = 11;

        private Panel panel;
        private Label[] label;
        private Label xlabel;
        private Label ylabel;
        private GraphicsPath gpAxis;
        private GraphicsPath[] gpZValue;

        // private members for x-axis and y-axis
        private int xstart = 75;
        private int ystart = 400;
        private int xlength = 512;              // relate to samplerate
        private int ylength = 360;
        // private members describing the data
        private double minZ = 0;
        private double maxZ = 0;
        private int dimX = 0;
        private int dimY = 0;
        // private members to display data
        private Color[] colorarray = new Color[numOfColors];

        public MyXYZDisplay()
        {
            StartPosition = FormStartPosition.CenterScreen;

            panel = new Panel();

            label = new Label[numOfLabels];
            for (int i=0; i<numOfLabels; i++)
            {
                label[i] = new Label();
            }
            xlabel = new Label();
            ylabel = new Label();

            gpZValue = new GraphicsPath[numOfColors];
            for (int i=0; i<numOfColors; i++)
            {
                gpZValue[i] = new GraphicsPath();
            }

            gpAxis = new GraphicsPath();

            // Panel for writing
            panel.Anchor = (AnchorStyles.Bottom | AnchorStyles.Top | AnchorStyles.Left | AnchorStyles.Right) ;
            panel.BackColor = SystemColors.Control;
            panel.Location = new Point(10, 10);
            panel.Size = new Size(700, 460);
            panel.Paint += new PaintEventHandler(OnPaint);

            // labels for the x-axis and y-axis
            for (int i=0; i<numOfLabels/2; i++)
            {
                // x-axis
                label[i].Location = new Point(xstart + i*xlength/4, ystart+20);
                label[i].Size = new Size(30, 15);
                label[i].Text = "0";
                label[i].Visible = true;
                // y-axis
                label[i+5].Location = new Point(xstart-25, ystart - i*ylength/4);
                label[i+5].Size = new Size(30, 15);
                label[i+5].Text = "0";
                label[i+5].Visible = true;
            }
            xlabel.Location = new Point(xstart+xlength/2, ystart+40);
            xlabel.Size = new Size(20, 20);
            xlabel.Font = new Font(xlabel.Font.FontFamily, 12);
            xlabel.Text = "f";
            xlabel.Visible = true;
            ylabel.Location = new Point(xstart-40, ystart-ylength/2);
            ylabel.Size = new Size(20, 20);
            ylabel.Font = new Font(ylabel.Font.FontFamily, 12);
            ylabel.Text = "k";
            ylabel.Visible = true;

            // Set up how the form should be displayed and add the controls to the form. 
            ClientSize = new Size(720, 480);
            Controls.AddRange(new Control[] {label[0], label[1], label[2], label[3], label[4], 
                                             label[5], label[6], label[7], label[8], label[9],
                                             xlabel, ylabel,
                                             panel});
            // fill the colorarray
            colorarray[10] = Color.DarkRed;
            colorarray[9] = Color.Red;
            colorarray[8] = Color.OrangeRed;
            colorarray[7] = Color.Orange;
            colorarray[6] = Color.Yellow;
            colorarray[5] = Color.GreenYellow;
            colorarray[4] = Color.LightGreen;
            colorarray[3] = Color.Green;
            colorarray[2] = Color.LightBlue;
            colorarray[1] = Color.Blue;
            colorarray[0] = Color.DarkBlue;

            // text of the main window
            Text = "Spectrogram";

            // draw the y-axis
            DrawLine(gpAxis, xstart, ystart, xstart, ystart-ylength);
            DrawLine(gpAxis, xstart-10, ystart-ylength, xstart, ystart-ylength);
            DrawLine(gpAxis, xstart-10, ystart-(1*ylength/4), xstart, ystart-(1*ylength/4));
            DrawLine(gpAxis, xstart-10, ystart-(2*ylength/4), xstart, ystart-(2*ylength/4));
            DrawLine(gpAxis, xstart-10, ystart-(3*ylength/4), xstart, ystart-(3*ylength/4));
            DrawLine(gpAxis, xstart-10, ystart, xstart, ystart);
            // draw the x-axis
            DrawLine(gpAxis, xstart, ystart, xstart+xlength, ystart);
            DrawLine(gpAxis, xstart, ystart+10, xstart, ystart);
            DrawLine(gpAxis, xstart+(1*xlength/4), ystart+10, xstart+(1*xlength/4), ystart);
            DrawLine(gpAxis, xstart+(2*xlength/4), ystart+10, xstart+(2*xlength/4), ystart);
            DrawLine(gpAxis, xstart+(3*xlength/4), ystart+10, xstart+(3*xlength/4), ystart);
            DrawLine(gpAxis, xstart+xlength, ystart+10, xstart+xlength, ystart);
        }

        private void DrawLine(GraphicsPath path, int x1, int y1, int x2, int y2)
        {
            path.StartFigure();
            path.AddLine(x1, y1, x2, y2);
            path.CloseFigure();
        }

        private void OnPaint(object sender, PaintEventArgs e)
        {
            Pen pen = new Pen(Color.Black, 1);
            e.Graphics.DrawPath(pen, gpAxis);

            for (int i=0; i<numOfColors; i++)
            {
                pen.Width = 4;
                pen.Color = colorarray[i];
                e.Graphics.DrawPath(pen, gpZValue[i]);
            }
        }

        public void AddData(double[,] data)
        {
            dimY = data.GetLength(0);
            dimX = data.GetLength(1);

            minZ = 1.0E10;
            maxZ = -1.0E10;

            // 1st loop to find min/max of z-values
            for (int i=0; i<dimY; i++)
            {
                for (int j=0; j<dimX; j++)
                {
                    if (data[i,j] > maxZ) maxZ = data[i,j];
                    if (data[i,j] < minZ) minZ = data[i,j];
                }
            }

            // 2nd loop to transform z-values into colors
            int index = 0;
            Rectangle rect = new Rectangle(0, 0, 0, 0);
            for (int i=0; i<dimY; i++)      // loop over all rows
            {
                for (int j=0; j<dimX; j++)  // loop over all columns
                {
                    // multi-color resolution
                    index = (int)((numOfColors-1)*(data[i, j]-minZ)/(maxZ-minZ));

                    //// 2-color resolution in yellow and darkblue (jag älskar sverige)
                    //if (data[i,j] <= (minZ+(maxZ-minZ)/2))
                    //{
                    //    index = 0;
                    //}
                    //else
                    //{
                    //    index = 6;
                    //}

                    rect = new Rectangle((int)(xstart + j * ((double)xlength / (double)dimX)),
                                         (int)(ystart - (i+1) * ((double)ylength / (double)dimY)),
                                         (int)((double)xlength / (double)dimX),
                                         (int)((double)ylength / (double)dimY));

                    // save the color-index for that specific rectangle
                    gpZValue[index].AddRectangle(rect);
                }
            }

            // draw the labels for the x-axis
            label[0].Text = "0";
            label[1].Text = (1*dimX/4).ToString();
            label[2].Text = (2*dimX/4).ToString();
            label[3].Text = (3*dimX/4).ToString();
            label[4].Text = (4*dimX/4).ToString();
            // draw the labels for the y-axis
            int multiplier = 2*dimX/dimY;
            label[5].Text = "0";
            label[6].Text = (multiplier*1*dimY/4).ToString();
            label[7].Text = (multiplier*2*dimY/4).ToString();
            label[8].Text = (multiplier*3*dimY/4).ToString();
            label[9].Text = (multiplier*4*dimY/4).ToString();
        }
    }
}
